"""
Define modeler entities
"""

from Accuinsight.modeler.entities.lc_run import Run
from Accuinsight.modeler.entities.lc_stage import LifecycleStage
from Accuinsight.modeler.entities.lc_metric import Metric
from Accuinsight.modeler.entities.lc_param import Param
from Accuinsight.modeler.entities.lc_run_status import RunStatus

from Accuinsight.modeler.entities.lc_run_info import LcRunInfo

__all__ = [
    "Run",
    "LifecycleStage",
    "Metric",
    "Param",
    "RunStatus",
    "LcRunInfo"
]


from Accuinsight.Lifecycle import keras
from Accuinsight.Lifecycle import tensorflow


class Lifecycle:
    """
    The exposed class to user
    """

    def run_keras(self, tag=None):
        keras.autolog(tag)

    def run_tensorflow(self, tag=None):
        tensorflow.autolog(tag)



import inspect
import json
from collections import OrderedDict
import warnings
import logging
from joblib import dump

from Accuinsight.modeler.core import func, path, get
from Accuinsight.modeler.core.func import get_time
from Accuinsight.modeler.core.get_for_visual import roc_pr_curve, get_true_y, get_visual_info_regressor
from Accuinsight.modeler.core.sklearnModelType import REGRESSION, CLASSIFICATION
from Accuinsight.modeler.core.LcConst.LcConst import ALL_MODEL_PARAMS, SELECTED_PARAMS, SELECTED_METRICS, VALUE_ERROR, \
    LOGGING_TIME, LOGGING_RUN_ID, FITTED_MODEL
from Accuinsight.modeler.core.Run.RunInfo.RunInfo import set_current_runs, clear_runs, \
    set_git_meta, set_python_dependencies, set_run_name, set_model_json_path, set_visual_json_path, set_model_file_path, set_prefix_path, set_best_model_joblib_path, _set_result_path, set_shap_json_path
from Accuinsight.modeler.utils.dependency.dependencies import gather_sources_and_dependencies
from Accuinsight.modeler.core.LcConst import LcConst
from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook, get_os_env
from Accuinsight.modeler.clients.modeler_api import LifecycleRestApi

from Accuinsight.modeler.core.feature_contribution import shap_value
from Accuinsight.modeler.core import metricsLightgbm
from Accuinsight.Lifecycle.common import Common

logging.basicConfig(level=logging.INFO,
                    format='%(message)s')

warnings.filterwarnings("ignore")


class accuinsight(Common):
    def __init__(self):
        super().__init__()

    def get_file(self, storage_json_file_name=None):
        global save_path, StorageInfo, target_name
        save_path, StorageInfo, target_name = super().get_file(storage_json_file_name)

    def set_features(self, feature_names):
        global feature_name
        feature_name = super().set_features(feature_names)

    class add_experiment(object):
        def __init__(self, model_name, *args, model_monitor=False):
            self.shap_on = model_monitor
            logging.info('Using add_experiment(model_monitor={})'.format(model_monitor))
            
            self.model_name = model_name
        
            # if in notebook
            if is_in_ipython():
                self.var_model_file_path = get_current_notebook()
                _caller_globals = inspect.stack()[1][0].f_globals
                (
                    self.mainfile,
                    self.sources,
                    self.dependencies
                ) = gather_sources_and_dependencies(
                    globs=_caller_globals,
                    save_git_info=False
                )
            else:
                _caller_globals = inspect.stack()[1][0].f_globals
                (
                    self.mainfile,
                    self.sources,
                    self.dependencies
                ) = gather_sources_and_dependencies(
                    globs=_caller_globals,
                    save_git_info=True
                )
                self.var_model_file_path = self.mainfile['filename']

            self.fitted_model = get.model_type(self.model_name)          # fitted model type
            self.json_path = OrderedDict()                               # path
            self.selected_params = []                                    # log params
            self.selected_metrics = OrderedDict()                        # log metrics
            self.summary_info = OrderedDict()                            # final results
            self.error_log = []                                          # error log
            self.vis_info = None                                         # visualization info - classifier
            self.dict_path = path.get_file_path(self.model_name)

            set_current_runs(get.model_type(self.model_name))
            _set_result_path(self.dict_path)

            # data for visualization function
            if len(args) == 2:
                self.xval = args[0]
                self.yval = args[1]

            elif len(args) == 4:
                self.xtrain = args[0]
                self.ytrain = args[1]
                self.xval = args[2]
                self.yval = args[3]
    
            else:
                raise ValueError('Check the arguments of function - add_experiment(model_name, X_val, y_val) or add_experiment(model_name, X_train, y_train, X_val, y_val)')
            
            # sklearn/xgboost/lightgbm
            get_from_model = get.from_model(self.model_name)
            self.all_model_params = get_from_model.all_params()
            self.model_param_keys = get_from_model.param_keys()
            
            # classifier
            if any(i in self.fitted_model for i in CLASSIFICATION):
                self.vis_info = roc_pr_curve(self.model_name, self.xval, self.yval)

            # regressor
            elif any(i in self.fitted_model for i in REGRESSION):
                self.ypred = get_visual_info_regressor(self.model_name, self.xval)
            
            # if user uses lightgbm package, verify the model for classification or regression.
            if self.fitted_model == 'lightgbm':
                
                # classifier
                if self.all_model_params['metric'] in metricsLightgbm.CLASSIFICATION:
                    self.vis_info = roc_pr_curve(self.model_name, self.xval, self.yval)
                
                # regressor
                elif self.all_model_params['metric'] in metricsLightgbm.REGRESSION:
                    self.ypred = get_visual_info_regressor(self.model_name, self.xval)
                
                # None
                elif self.all_model_params['metric'] in metricsLightgbm.no_metrics:
                    raise ValueError('Please set any metric parameter in the lightGBM.')
            
            set_model_file_path(self.var_model_file_path)

            if hasattr(self, 'mainfile'):
                set_git_meta(fileinfo=self.mainfile)
            if hasattr(self, 'dependencies'):
                set_python_dependencies(py_depenpency=self.dependencies)

        def __enter__(self):
            self.start_time = get_time.now()
            self.summary_info[LOGGING_TIME] = get_time.logging_time()
            self.summary_info[LOGGING_RUN_ID] = func.get_run_id()
            self.summary_info[FITTED_MODEL] = self.fitted_model

            set_prefix_path(self.dict_path[LcConst.RUN_PREFIX_PATH])
            set_run_name(self.fitted_model, self.summary_info[LOGGING_RUN_ID])

            return self

        def __exit__(self, a, b, c):
            self.summary_info[ALL_MODEL_PARAMS] = self.all_model_params
            self.summary_info[SELECTED_PARAMS] = self.selected_params
            self.summary_info[SELECTED_METRICS] = self.selected_metrics
            self.summary_info[VALUE_ERROR] = self.error_log

            # model_monitor = True
            self.run_id = self.summary_info[LOGGING_RUN_ID]
            
            if self.shap_on:
                if not feature_name:
                    try:
                        self.feature_name = get.feature_name(save_path, StorageInfo, target_name)
                    except:
                        self.feature_name = None
                else:
                    self.feature_name = feature_name

                self.run_id = self.fitted_model + '-' + self.run_id
                
                # when user uses lightGBM fitted by lightGBM datasets, convert data type to lightGBM dataset.
                if self.fitted_model == 'lightgbm':
                    self.xtrain = lgb.Dataset(self.xtrain, label=self.ytrain, silent=True)
                        
                self.shap_value = shap_value(self.model_name, self.xtrain, self.feature_name)
                
                # path for shap.json
                shap_json_full_path = self.dict_path['shap_json_full']
                set_shap_json_path(self.dict_path['shap_json'])
                
                with open(shap_json_full_path, 'w', encoding = 'utf-8') as save_file:
                    json.dump(self.shap_value, save_file, indent='\t')
                
            else:
                pass
                
            # visualization
            if any(i in self.fitted_model for i in CLASSIFICATION) or self.all_model_params['metric'] in metricsLightgbm.CLASSIFICATION:

                # path for visual.json
                visual_json_full_path = self.dict_path['visual_json_full']
                set_visual_json_path(self.dict_path['visual_json'])

                with open(visual_json_full_path, 'w', encoding='utf-8') as save_file1:
                    json.dump(self.vis_info, save_file1, indent="\t")

            elif any(i in self.fitted_model for i in REGRESSION) or self.all_model_params['metric'] in metricsLightgbm.REGRESSION:
                temp_yval = get_true_y(self.yval)
                if len(temp_yval) <= 5000:
                    self.summary_info['True_y'] = temp_yval
                    self.summary_info['Predicted_y'] = get_visual_info_regressor(self.model_name, self.xval)
                else:
                    self.summary_info['True_y'] = None
                    self.summary_info['Predicted_y'] = None

            self.summary_info['ValueError'] = self.error_log

            if not self.summary_info['ValueError']:
                # path for model_info.json
                model_json_full_path = self.dict_path['model_json_full']
                set_model_json_path(self.dict_path['model_json'])

                with open(model_json_full_path, 'w', encoding='utf-8') as save_file2:
                    json.dump(self.summary_info, save_file2, indent="\t")
                    
            else:
                pass

            # model save
            save_model_path = self.dict_path['save_model_joblib'] + self.summary_info[FITTED_MODEL] + '-' + self.summary_info[LOGGING_RUN_ID] +'.joblib'
            path_for_setting_model_joblib = self.dict_path['save_model_dir'] + '/' + self.summary_info[FITTED_MODEL] + '-' + self.summary_info[LOGGING_RUN_ID] +'.joblib'

            set_best_model_joblib_path(path_for_setting_model_joblib)

            dump(self.model_name, save_model_path)

            start_time = int(self.start_time.timestamp()*1000)
            end_time = int(get_time.now().timestamp()*1000)
            delta_ts = end_time - start_time

            clear_runs(start_time, end_time, delta_ts)
            env_value = get_os_env('ENV')
            modeler_rest = LifecycleRestApi(env_value[LcConst.BACK_END_API_URL],
                                            env_value[LcConst.BACK_END_API_PORT],
                                            env_value[LcConst.BACK_END_API_URI])
            modeler_rest.lc_create_run()

        def log_params(self, param = None):
            # sklearn/xgboost/lightgbm
            if param:
                if param in self.model_param_keys:
                    return self.selected_params.append(param)

                else:
                    self.error_log.append(True)
                    raise ValueError('"' + param + '"' + ' does not exist in the model.')

        def log_metrics(self, metric_name, defined_metric):
            self.selected_metrics[metric_name] = defined_metric

        def log_tag(self, description):
            self.summary_info['tag'] = description



import uuid
import datetime

def get_run_id():
    random_id = str(uuid.uuid4()) 
    random_id = random_id.upper()
    return(random_id)

class get_time(object):
    def now():
        return(datetime.datetime.now())
    def logging_time():
        dt = datetime.datetime.now()
        return(dt.strftime('%Y-%m-%d %H:%M:%S'))



import os
from collections.abc import Mapping
from csv import DictReader
import pandas as pd
from pathlib import Path
import json
from Accuinsight.modeler.core.Run.RunInfo.RunInfo import get_current_run
from Accuinsight.modeler.core.LcConst.LcConst import RUN_OBJ_NAME, RUN_OBJ_JSON_PATH, RUN_OBJ_CSV_PATH, RUN_ROOT_PATH, \
    RUN_BASE_PATH, RUN_RESULT_PATH, RUN_MODEL_VISUAL_CSV_PATH, RUN_PREFIX_PATH, RUN_MODEL_JSON_PATH
from Accuinsight.modeler.protos.life_cycle_pb2 import LcMetric, LcParam

# get parameter path with run type
# get metric path with run type


def _get_current_run():
    # {'run_name': 'keras',
    #  'json_path': 'model-info-json/keras_nn-1.json',
    #  'csv_path': 'for-visual-csv/keras_nn-history-1.csv'}

    return get_current_run()


def load_keras_parameter():
    run_info_json = _get_current_run()

    run_result_path = run_info_json[RUN_RESULT_PATH]
    base_path = run_result_path[RUN_BASE_PATH]
    prefix_path = run_result_path[RUN_PREFIX_PATH]
    json_path = run_result_path[RUN_MODEL_JSON_PATH]
    param_path = ''.join([base_path, prefix_path, json_path])

    with open(param_path) as json_file:
        param_json_data = json.load(json_file)

    return param_json_data


def _read_csv_with_pandas():
    run_info_json = _get_current_run()
    csv_path = run_info_json[RUN_BASE_PATH] + 'results-keras/' + run_info_json[RUN_OBJ_CSV_PATH]
    # csv_data = pd.read_csv(csv_path,
    #                        sep=';', index_col='epoch')

    csv_data = pd.read_csv(csv_path, sep=';', index_col=0, squeeze=True, header=None).to_dict()
    return csv_data


def _read_csv_with_csv():
    column_dict = {}

    run_info_json = _get_current_run()

    run_result_path = run_info_json[RUN_RESULT_PATH]
    base_path = run_result_path[RUN_BASE_PATH]
    prefix_path = run_result_path[RUN_PREFIX_PATH]
    csv_path = run_result_path[RUN_MODEL_VISUAL_CSV_PATH]

    csv_path = ''.join([base_path, prefix_path, csv_path])

    with open(csv_path, 'r') as read_csv:
        csv_dict_reader = DictReader(read_csv, delimiter=';')
        column_names = csv_dict_reader.fieldnames

        for row in csv_dict_reader:
            for col_name in column_names:
                column_dict.setdefault(col_name, []).append(row[col_name])

    return column_dict


def load_keras_metric():
    # csv_data = _read_csv_with_pandas()
    column_dict = _read_csv_with_csv()

    result_dict = {'metrics': []}
    for key in column_dict:
        if key != 'epoch':
            result_dict['metrics'].append({'key': key, 'values': column_dict[key], 'timestamp': column_dict['epoch'], 'steps': column_dict['epoch']})

    return result_dict


def parse_keras_metric():
    metric_data = load_keras_metric()
    metrics = metric_data['metrics']

    grpc_metrics = []

    for item in metrics:
        metric = LcMetric(
                key=item['key'],
                values=item['values'],
                timestamp=item['timestamp'],
                steps=item['steps']
            )
        grpc_metrics.append(metric)
    return grpc_metrics


def _parse_optimizer_info(param_data):
    grpc_params = []
    optimizer_info = param_data['optimizer_info']
    if isinstance(optimizer_info, Mapping):
        for k, v in optimizer_info.items():
            optimizer = LcParam(
                key='type_of_optimizer',
                value=str(k)
            )
            grpc_params.append(optimizer)
            if isinstance(v, Mapping):
                for k, v in v.items():
                    param = LcParam(
                        key=k,
                        value=str(v)
                    )
                    grpc_params.append(param)
    return grpc_params


def parse_keras_parameter():
    param_data = load_keras_parameter()

    dummy_data = {
        'model_description': 'keras test',
        'logging_time': '2020-05-06 10:30:50',
        'run_id': '9FCEF776-D830-4353-8961-144491DC03CC',
        'model_type': 'keras_nn',
        'optimizer_info': {
            'RMSprop': {
                'learning_rate': 9.999999747378752e-05,
                'rho': 0.8999999761581421,
                'decay': 0.0,
                'epsilon': 1e-07
            }
        },
        'time_delta': '0:00:14.912202'
    }

    return _parse_optimizer_info(param_data)


def parse_run_result():
    result_dict = {'metrics': None, 'params': None, 'visual': None}

    metric_data = parse_keras_metric()
    parameter_data = parse_keras_parameter()

    result_dict['params'] = parameter_data
    result_dict['metrics'] = metric_data
    return result_dict




from collections import OrderedDict
import numpy as np
from scipy import interpolate

def delete_nan(lst):
    if np.isnan(lst).any():
        idx = np.where(np.isnan(lst))[0].tolist()

        for i in idx:
            lst.pop(i)
    
    return(lst)
                       
def sort(x):
    sort_result = np.sort(x)[::-1]
    sort_idx = x.argsort()[::-1]
    return({'x': sort_result, 'idx':sort_idx})

def moving_average(a, n) :
    cumsum = np.cumsum(a, dtype=float)
    cumsum[n:] = cumsum[n:] - cumsum[:-n]
    return cumsum[n - 1:] / n

def cal_confusion(y_test_vec, y_score_vec, force_diag = True):
    results = OrderedDict()
    y_score_vec_sort = sort(y_score_vec)
    y_score_vec = y_score_vec_sort['x']
    y_test_vec = y_test_vec[y_score_vec_sort['idx']]
    
    results['TP'] = np.cumsum(y_test_vec==1)
    results['FP'] = np.cumsum(y_test_vec==0)
    results['FN'] = np.sum(y_test_vec==1) - results['TP']
    results['TN'] = np.sum(y_test_vec==0) - results['FP']
    
    results['TPR'] = results['TP']/(results['TP'] + results['FN'])
    results['FPR'] = results['FP']/(results['FP'] + results['TN'])
    results['PPV'] = results['TP']/(results['TP'] + results['FP'])
    
    if force_diag:
        results['TPR'] = np.r_[np.array(0), results['TPR']]
        results['FPR'] = np.r_[np.array(0), results['FPR']]
        results['PPV'] = np.r_[results['PPV'][0], results['PPV']]
    return(results)

def cal_auc(tpr, ppv):
    idx = tpr.argsort(kind='stable')
    auc = np.sum(np.diff(tpr[idx]) * moving_average(ppv[idx], 2))
    return(auc)

def multi_pr(y_test, y_score):
    recall = OrderedDict()
    precision = OrderedDict()
    auc = OrderedDict()

    class_names = np.array(range(y_test.shape[0]))
    # press data
    press_num = 1000

    for i in range(len(class_names)):
        y_test_vec = np.array(y_test[i])
        y_score_vec = np.array(y_score[i])
    
        if len(y_test_vec) >= 2000:
            p = int(np.around(len(y_test_vec)/press_num))
        else:
            p = 1

        confus = cal_confusion(y_test_vec, y_score_vec, force_diag = True)
        recall[i] = confus['TPR'][::p].tolist()
        precision[i] = confus['PPV'][::p].tolist()
        auc[i] = cal_auc(confus['TPR'][::p], confus['PPV'][::p])
    
    all_recall = np.sort(np.unique(np.array([recall[i] for i in range(len(recall))]).flatten()))[::-1]
    all_precision = np.zeros_like(all_recall)

    for i in range(len(class_names)):
        f = interpolate.interp1d(recall[i], precision[i], kind='linear', fill_value="extrapolate")
        all_precision = all_precision + f(all_recall)

    all_precision = all_precision/len(class_names)

    if len(all_recall) >= 2000:
            p = int(np.around(len(all_recall)/press_num))
    else:
            p = 1

    recall['macro'] = all_recall[::p].tolist()
    precision['macro'] = all_precision[::p].tolist()
    auc['macro'] = cal_auc(all_recall[::p], all_precision[::p])

    y_test_vec_bin = y_test.flatten()
    y_score_vec_bin = y_score.flatten()
    confus_bin = cal_confusion(y_test_vec_bin, y_score_vec_bin)

    if len(y_test_vec_bin) >= 2000:
        p = int(np.around(len(y_test_vec_bin)/press_num))
    else:
        p = 1

    recall['micro'] = confus_bin['TPR'][::p].tolist()
    precision['micro'] = confus_bin['PPV'][::p].tolist()
    auc['micro'] = cal_auc(confus_bin['TPR'][::p], confus_bin['PPV'][::p])
    
    
    # nan 값이 나오더라도 json 저장 가능하도록
    delete_nan(recall['macro'])
    delete_nan(precision['macro'])
    delete_nan(recall['micro'])
    delete_nan(precision['micro'])
    
    if str(auc['macro']) == 'nan':
        auc['macro'] = int(0)
    if str(auc['micro']) == 'nan':
        auc['micro'] = int(0)
    
    
    res_dict = OrderedDict()
    if len(class_names) <= 5:
        res_dict['recall'] = recall
        res_dict['precision'] = precision
        res_dict['auc'] = auc
    else:
        res_dict['recall'] = {'macro': recall['macro']}
        res_dict['recall']['micro'] = recall['micro']
        res_dict['precision'] = {'macro': precision['macro']}
        res_dict['precision']['micro'] = precision['micro']
        res_dict['auc'] = {'macro': auc['macro']}
        res_dict['auc']['micro'] = auc['micro']
    
    return(res_dict)

def multi_roc(y_test, y_score):
    fpr = OrderedDict()
    tpr = OrderedDict()
    auc = OrderedDict()

    class_names = np.array(range(y_test.shape[0]))

    # press data
    press_num = 1000

    for i in range(len(class_names)):
        y_test_vec = np.array(y_test[i])
        y_score_vec = np.array(y_score[i])
    
        if len(y_test_vec) >= 2000:
            p = int(np.around(len(y_test_vec)/press_num))
        else:
            p = 1

        confus = cal_confusion(y_test_vec, y_score_vec, force_diag = True)
        fpr[i] = confus['FPR'][::p].tolist()
        tpr[i] = confus['TPR'][::p].tolist()
        auc[i] = cal_auc(1-confus['FPR'][::p], confus['TPR'][::p])
    
    all_fpr = np.sort(np.unique(np.array([fpr[i] for i in range(len(fpr))]).flatten()))[::-1]
    all_tpr = np.zeros_like(all_fpr)

    for i in range(len(class_names)):
        f = interpolate.interp1d(fpr[i], tpr[i], kind='linear')
        all_tpr = all_tpr + f(all_fpr)

    all_tpr = all_tpr/len(class_names)

    if len(all_fpr) >= 2000:
            p = int(np.around(len(all_fpr)/press_num))
    else:
            p = 1

    fpr['macro'] = all_fpr[::p].tolist()
    tpr['macro'] = all_tpr[::p].tolist()
    auc['macro'] = cal_auc(all_fpr[::p], all_tpr[::p])

    y_test_vec_bin = y_test.flatten()
    y_score_vec_bin = y_score.flatten()
    confus_bin = cal_confusion(y_test_vec_bin, y_score_vec_bin)

    if len(y_test_vec_bin) >= 2000:
        p = int(np.around(len(y_test_vec_bin)/press_num))
    else:
        p = 1

    fpr['micro'] = confus_bin['FPR'][::p].tolist()
    tpr['micro'] = confus_bin['TPR'][::p].tolist()
    auc['micro'] = cal_auc(confus_bin['FPR'][::p], confus_bin['TPR'][::p])
    
    # nan 값이 나오더라도 json 저장 가능하도록
    delete_nan(fpr['macro'])
    delete_nan(tpr['macro'])
    delete_nan(fpr['micro'])
    delete_nan(tpr['micro'])
    
    if str(auc['macro']) == 'nan':
        auc['macro'] = int(0)
    if str(auc['micro']) == 'nan':
        auc['micro'] = int(0)
    
    res_dict = OrderedDict()
    if len(class_names) <= 5:
        res_dict['fpr'] = fpr
        res_dict['tpr'] = tpr
        res_dict['auc'] = auc
    else:
        res_dict['fpr'] = {'macro': fpr['macro']}
        res_dict['fpr']['micro'] = fpr['micro']
        res_dict['tpr'] = {'macro': tpr['macro']}
        res_dict['tpr']['micro'] = tpr['micro']
        res_dict['auc'] = {'macro': auc['macro']}
        res_dict['auc']['micro'] = auc['micro']
    
    return(res_dict)




#!/usr/bin/env python
# coding=utf-8

import platform
from munch import munchify

__all__ = ("SETTINGS",)


SETTINGS = munchify(
    {
        "CONFIG": {
            # make sure all config keys are compatible with MongoDB
            "ENFORCE_KEYS_MONGO_COMPATIBLE": True,
            # make sure all config keys are serializable with jsonpickle
            # THIS IS IMPORTANT. Only deactivate if you know what you're doing.
            "ENFORCE_KEYS_JSONPICKLE_COMPATIBLE": True,
            # make sure all config keys are valid python identifiers
            "ENFORCE_VALID_PYTHON_IDENTIFIER_KEYS": False,
            # make sure all config keys are strings
            "ENFORCE_STRING_KEYS": False,
            # make sure no config key contains an equals sign
            "ENFORCE_KEYS_NO_EQUALS": True,
            # if true, all dicts and lists in the configuration of a captured
            # function are replaced with a read-only container that raises an
            # Exception if it is attempted to write to those containers
            "READ_ONLY_CONFIG": True,
            # regex patterns to filter out certain IDE or linter directives from
            # inline comments in the documentation
            "IGNORED_COMMENTS": ["^pylint:", "^noinspection"],
        },
        "HOST_INFO": {
            # Collect information about GPUs using the nvidia-smi tool
            "INCLUDE_GPU_INFO": True,
            # List of ENVIRONMENT variables to store in host-info
            "CAPTURED_ENV": [],
        },
        "COMMAND_LINE": {
            # disallow string fallback, if parsing a value from command-line failed
            "STRICT_PARSING": False,
            # show command line options that are disabled (e.g. unmet dependencies)
            "SHOW_DISABLED_OPTIONS": True,
        },
        # configure how stdout/stderr are captured. ['no', 'sys', 'fd']
        "CAPTURE_MODE": "sys" if platform.system() == "Windows" else "fd",
        # configure how dependencies are discovered. [none, imported, sys, pkg]
        "DISCOVER_DEPENDENCIES": "imported",
        # configure how source-files are discovered. [none, imported, sys, dir]
        "DISCOVER_SOURCES": "imported",
    }
)
